﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using brBaseDLL;
using brPerfDLL;


namespace Performancetest
{
    public partial class Form1 : Form
    {

        //Variable
        TextWriter _writer = null;
        Thread cmd_thread = null;
        Boolean thread_running = false;

    
        public Form1()
        {
            InitializeComponent();
            _writer = new TextBoxStreamWriter(textBox_terminal);
            Console.SetOut(_writer);
            comboBox_speed.SelectedIndex = 0;
        }


        //Downstreamtest
        private void button_startdownstream_Click(object sender, EventArgs e)
        {
            //interface
            String io_interface;
            if (radioButton_USB.Checked)
                io_interface = "usb";
            else
                io_interface = textBox_ip.Text;
            //speed index
            byte speed = (byte)comboBox_speed.SelectedIndex;
            //duration
            UInt16 duration = (UInt16)numericUpDown_ms.Value;
            //start thread
            if (thread_running)
            {
                Console.WriteLine("... another test is still running ...");
                return;
            }
            thread_running = true;
            Thread.Sleep(100);
            cmd_thread = new Thread(delegate ()
            {
                //set testspeed
                if (brPerf.ctrl_setTestGenSpeed(io_interface, speed, false))
                {
                    Console.Write("... start download -> ");
                    //run test
                    if (brBase.stream_readData_ms(io_interface, duration, "tempfile.bit", false))
                    {
                        FileInfo info = new FileInfo("tempfile.bit");
                        if (speed < 8)
                            Console.WriteLine("Performance: " + Math.Round(info.Length / (double)duration / 1000, 3).ToString().PadLeft(8, ' ') + " MBps");
                        else
                            Console.WriteLine("Performance: " + Math.Round(info.Length / (double)duration, 3).ToString().PadLeft(8, ' ') + " kBps");
                        Console.Write("... check for counter errors -> ");
                        Console.WriteLine("Errors: " + checkTestGenData("tempfile.bit"));
                    }
                    Console.WriteLine();
                    //delete temporary file
                    if (File.Exists("tempfile.bit")) File.Delete("tempfile.bit");
                }
                //reset flag
                thread_running = false;
            });
            cmd_thread.Start();
        }


        //check received testgenerator data
        public int checkTestGenData(String tempfile)
        {
            //Variable
            UInt16 wert1 = 0, wert2 = 0, wert3 = 0, wert4 = 0;
            int part = 0;
            UInt16 fehler_down = 0, x = 0;
            if (File.Exists(tempfile))
            {
                FileStream stream = new FileStream(tempfile, FileMode.Open, FileAccess.Read);
                BinaryReader reader = new BinaryReader(stream);
                int arraylength = 1000;
                byte[] file_array = new byte[arraylength];
                part = reader.Read(file_array, 0, 8);
                if (part != 8) return -1;
                byte b0 = file_array[0];
                byte b1 = file_array[1];
                byte b2 = file_array[2];
                byte b3 = file_array[3];
                byte b4 = file_array[4];
                byte b5 = file_array[5];
                byte b6 = file_array[6];
                byte b7 = file_array[7];
                x = 8;
                wert1 = (UInt16)(((UInt16)b0 << 8) | (UInt16)b1);
                wert2 = (UInt16)(((UInt16)b2 << 8) | (UInt16)b3);
                wert3 = (UInt16)(((UInt16)b4 << 8) | (UInt16)b5);
                wert4 = (UInt16)(((UInt16)b6 << 8) | (UInt16)b7);
                while (wert2 != (wert1 + 1) % Math.Pow(2, 16) || wert3 != (wert2 + 1) % Math.Pow(2, 16) || wert4 != (wert3 + 1) % Math.Pow(2, 16))
                {
                   b0 = b1;
                    b1 = b2;
                    b2 = b3;
                    b3 = b4;
                    b4 = b5;
                    b5 = b6;
                    b6 = b7;
                    part = reader.Read(file_array, 0, 1);
                    if (part != 1)
                    {
                        reader.Close();
                        stream.Close();
                        return -1;
                    }
                    x++;
                    b7 = file_array[0];
                    wert1 = (UInt16)(((UInt16)b0 << 8) | (UInt16)b1);
                    wert2 = (UInt16)(((UInt16)b2 << 8) | (UInt16)b3);
                    wert3 = (UInt16)(((UInt16)b4 << 8) | (UInt16)b5);
                    wert4 = (UInt16)(((UInt16)b6 << 8) | (UInt16)b7);
                }
                wert2 = wert4;
                do
                {
                    part = reader.Read(file_array, 0, arraylength);
                    if (part > 0)
                    {
                        wert1 = (UInt16)(((UInt16)file_array[0] << 8) | (UInt16)file_array[1]);
                        if (wert1 != (wert2 + 1) % Math.Pow(2, 16)) fehler_down++;
                        for (int k = 0; k < part - 3; k += 2)
                        {
                            wert1 = (UInt16)(((UInt16)file_array[k] << 8) | (UInt16)file_array[k + 1]);
                            wert2 = (UInt16)(((UInt16)file_array[k + 2] << 8) | (UInt16)file_array[k + 3]);
                            if (wert2 != (wert1 + 1) % Math.Pow(2, 16)) fehler_down++;
                        }
                    }
                } while (part > 0);
                reader.Close();
                stream.Close();
            }
            else
            {
                Console.WriteLine("... Fehler in der Testdatei");
                return -1;
            }
            return fehler_down;
        }


        //Upstreamtest
        private void button_startupstream_Click(object sender, EventArgs e)
        {
            //interface
            String io_interface;
            if (radioButton_USB.Checked)
                io_interface = "usb";
            else
                io_interface = textBox_ip.Text;
            //speed index
            byte speed = (byte)comboBox_speed.SelectedIndex;
            //duration
            UInt16 duration = (UInt16)numericUpDown_ms.Value;
            //start thread
            if (thread_running)
            {
                Console.WriteLine("... another test is still running ...");
                return;
            }
            thread_running = true;
            Thread.Sleep(100);
            cmd_thread = new Thread(delegate ()
            {
                //set testspeed
                if (brPerf.ctrl_setTestGenSpeed(io_interface, speed, false))
                {
                    Console.WriteLine("... please wait - need to download testgenerator data first ... ");
                    //run download
                    if (brBase.stream_readData_ms(io_interface, (UInt16)(1.5*duration), "tempfile.bit", false))
                    {
                        //reset
                        if (brPerf.ctrl_resetUploadValues(io_interface, false))
                        {
                            Console.Write("... start upload -> ");
                            //run test
                            if (brBase.stream_writeData_ms(io_interface, duration, "tempfile.bit", false))
                            {
                                UInt32 bytes, errors;
                                Boolean found_counter;
                                //read upload test values
                                if (brPerf.ctrl_readUploadValues(io_interface, out found_counter, out bytes, out errors, false))
                                {                                    
                                    if (speed < 8)
                                        Console.WriteLine("Performance: " + Math.Round(bytes / (double)duration / 1000, 3).ToString().PadLeft(8, ' ') + " MBps");
                                    else
                                        Console.WriteLine("Performance: " + Math.Round(bytes / (double)duration, 3).ToString().PadLeft(8, ' ') + " kBps");
                                    Console.Write("... check for counter errors -> ");
                                    Console.WriteLine("Errors: " + errors);
                                    if (!found_counter) Console.WriteLine("... NO counter detected");
                                }
                            }
                        }
                    }
                    Console.WriteLine();
                    //delete temporary file
                    if (File.Exists("tempfile.bit")) File.Delete("tempfile.bit");
                }
                //reset flag
                thread_running = false;
            });
            cmd_thread.Start();
        }

    }
}
